/*
 *  The Apache Software License, Version 1.1
 *
 *  Copyright (c) 2001-2002 The Apache Software Foundation.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  3. The end-user documentation included with the redistribution, if
 *  any, must include the following acknowlegement:
 *  "This product includes software developed by the
 *  Apache Software Foundation (http://www.apache.org/)."
 *  Alternately, this acknowlegement may appear in the software itself,
 *  if and wherever such third-party acknowlegements normally appear.
 *
 *  4. The names "The Jakarta Project", "Ant", and "Apache Software
 *  Foundation" must not be used to endorse or promote products derived
 *  from this software without prior written permission. For written
 *  permission, please contact apache@apache.org.
 *
 *  5. Products derived from this software may not be called "Apache"
 *  nor may "Apache" appear in their names without prior written
 *  permission of the Apache Group.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *  ====================================================================
 *
 *  This software consists of voluntary contributions made by many
 *  individuals on behalf of the Apache Software Foundation.  For more
 *  information on the Apache Software Foundation, please see
 *  <http://www.apache.org/>.
 */
package org.apache.tools.ant.taskdefs.optional.ssh;

import org.apache.tools.ant.taskdefs.optional.ssh.SSHUserInfo;

import org.apache.tools.ant.Task;
import org.apache.tools.ant.TaskContainer;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.LogOutputStream;


import java.io.*;
import java.io.ByteArrayOutputStream;
import java.util.Vector;
import java.util.Enumeration;

import java.io.BufferedReader;
import java.io.InputStreamReader;

import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;

/**
 * Executes a command on a remote machine via ssh.
 * @author    Robert Anderson, riznob@hotmail.com
 * @created   February 2, 2003
 */
public class SSHExec extends SSHBase {

    private int port = 22;
    private String command = null;
    private int maxwait = 30000;

	/**
	 * Constructor for SSHExecTask.
	 */
	public SSHExec() {
		super();
	}

    /**
     * Remote host, either DNS name or IP.
     *
     * @param host  The new host value
     */
    public void setHost(String host) {
        this.host = host;
    }
	
    /**
     * Sets the command to execute on the remote host.
     *
     * @param command  The new command value
     */
    public void setCommand(String command) {
        this.command = command;
    }

    /**
     * The connection will be dropped after maxwait seconds. This is 
     * sometimes useful when a connection may be flaky. Default is to 
     * wait forever.
     *
     * @param maxwait  The new maxwait value
     */
    public void setMaxwait(int maxwait) {
        this.maxwait = maxwait;
    }


    /**
     * Execute the command on the remote host.
     * @exception BuildException  Most likely a network error or bad
     * parameter.
     */
    public void execute() throws BuildException {
        if (host == null) {
            throw new BuildException("Host is null.");
        }
        if (userInfo.getName() == null) {
            throw new BuildException("Username is null.");
        }
        if (userInfo.getKeyfile() == null & userInfo.getPassword() == null) {
            throw new BuildException("Password and Keyfile are null.");
        }
        if (command == null) {
            throw new BuildException("Command is null.");
        }

		Session session = null;
		try {
		JSch jsch=new JSch();
		if (null != userInfo.getKeyfile()) {
			jsch.addIdentity(userInfo.getKeyfile(), "passphrase");
		}
		session = jsch.getSession(userInfo.getName(), host, port);
		session.setUserInfo(userInfo);
		session.connect();
		ChannelExec channel=(ChannelExec) session.openChannel("exec");
		channel.setCommand(command);
		channel.setInputStream(System.in);
		channel.setOutputStream(System.out);
		channel.connect();
		}
		catch(Exception e){
			throw new BuildException(e);
		}
    }
}

